'use strict';

var prefixMsg = function prefixMsg(str) {
  return "[enumStatus] " + str;
};

var warn = function warn(str) {
  console.log(prefixMsg(str));
  console.trace();
};

var typeError = function typeError(str) {
  var msg = prefixMsg(str);
  throw new TypeError(msg);
};

var _extends$1 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var extend = (function (base) {
  return function (extension) {
    return _extends$1({}, base, extension, {
      id: base.id
    });
  };
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var createIs = function createIs(names, target) {
  var _names$reduce = names.reduce(function (_ref, name) {
    var is = _ref.is,
        not = _ref.not;

    is = _extends({}, is, _defineProperty({}, name, name === target));
    not = _extends({}, not, _defineProperty({}, name, !is[name]));
    return { is: is, not: not };
  }, { is: {}, not: {} }),
      is = _names$reduce.is,
      not = _names$reduce.not;

  is.not = not;

  return is;
};

var isLowercase = function isLowercase(str) {
  return (/^[a-z_]+$/.test(str)
  );
};

var validate = function validate(table) {
  var names = Object.keys(table);

  // NOTE: should i warn?
  // if (!names.length) {
  //   warn('no status is defined, are you sure?');
  // }

  if (names.some(isLowercase)) {
    warn('Use uppercase for status names');
  }

  names.forEach(function (name) {
    if (!('id' in table[name])) {
      typeError(name + ' state have not prop id');
    }
  });
};

var createState = function createState(options, name, names) {
  return _extends({}, options, {
    is: createIs(names, name)
  });
};

var index = (function (table) {
  validate(table);

  var names = Object.keys(table);
  var idMap = {};
  var nameMap = {};
  var enums = [];
  var defaultValue = undefined;

  names.forEach(function (name) {
    var state = createState(table[name], name, names);
    state.extend = extend(state);

    idMap[state.id] = state;
    nameMap[name] = state;
    enums.push(state);
  });

  var status = _extends({}, nameMap, {
    getById: function getById(id) {
      return idMap[id] || defaultValue;
    },
    enums: enums,
    default: function _default(value) {
      if (!(value in nameMap)) {
        typeError('default value name(' + value + ') is undefined');
      }

      defaultValue = nameMap[value];

      return status;
    }
  });

  return status;
});

module.exports = index;
